--Alta Central Produccin--
/*Procedimiento que a partir de los parmetros de entrada aade un nueva central de Produccin a la base de datos
 *PARAMETROS IN: codigo (variable tipo varchar2, contendr el identificador nico de la central)
 *		 tipo (varible tipo varchar2, contendr el tipo (nuclear, eolica, solar, etc..) de central.
 *		 dir (variable de tipo Varchar2, contiene la direccin(calle, avenida, polgono, etc). de la central)
 *		 loc (variable de tipo varchar2, contiene la localidad (Barcelona, Madrid, etc) de la central)
 *		 ps  (Variable de tipo varchar2, contiene el pas donde est situada la central)
 *		 consumo_func (Variable de tipo NUMBER, contiene la energa que la central necesita consumir para funcionar)
 *		 residuos_rad (variable de tipo NUMBER, contiene el volumen de residuos radiactivos generados por las nucleares)
 *		 emisiones_co2 (Variable de tipo NUMBER, contiene los KG de emisin de CO2 generados por las centrales Termica y Carbn)
 *	 	 molinos (Variable de tipo NUMBER, contiene el nmero de molinos instalados en una central Elica.
 *		 paneles (Variable de tipo NUMBER, contiene el nmero de paneles que necesita una central Solar para funcionar)
 *		 ultima_inspeccion (Variable de tipo DATE, contiene la fecha de la ltima inspeccin de la Central). 
 *		 RSP (Variable de tipo CLOB que se corresponde con el parmetro de retorno)
 *RETURN: Parmetro RSP, variable de tipo CLOB que contendr informacin sobre el resultado de la ejecucin del procedimiento.
 */

CREATE OR REPLACE
PROCEDURE pr_alta_central_p(codigo IN VARCHAR2, tipo IN VARCHAR2, dir IN VARCHAR2,loc IN VARCHAR2, ps IN VARCHAR2, consumo_func IN NUMBER,
residuos_rad IN NUMBER,emisiones_co2 IN NUMBER, molinos IN NUMBER, paneles IN NUMBER, ultima_inspeccion IN DATE, RSP OUT NOCOPY CLOB) AS
    
    --Declaracin de variables 
    fecha_proceso VARCHAR2(20);
    nombre_proceso VARCHAR2(20);
    In_log CLOB;
    existe INTEGER;
    
    --Declaracin de excepciones
    EX_CENTRAL_EXISTE EXCEPTION;
    EX_CENTRAL_TIPO_NULL EXCEPTION;
    EX_CENTRAL_DIRECCION_NULL EXCEPTION;
    EX_CENTRAL_LOCALIDAD_NULL EXCEPTION;
    EX_CENTRAL_PAIS_NULL EXCEPTION;
    EX_VALORES_NULOS_TIPO EXCEPTION;
      
    --Ejecucin del procedimiento.
    BEGIN
         
        --obtenemos la fecha de ejecucin del procedimiento a partir de los datos del sistema
	fecha_proceso:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
        nombre_proceso:='pr_alta_central_p';
      
        --Creamos una cadena de texto con los parametros de entrada separados por barras.
        In_log:= (codigo||'  ||  ' || tipo ||'  ||  ' || dir||'  ||  ' || loc||'  ||  ' || ps ||'  ||  '|| consumo_func ||'  ||  ' || residuos_rad||'  ||  ' || emisiones_co2 ||'  ||  ' || molinos||'  ||  ' || paneles||'  ||  ' || ultima_inspeccion);        
        
	--Comprobamos que no existe una central con la misma identificacin lo cual
        --supondra una violacin de la PK.
        SELECT COUNT(*)INTO existe FROM tb_Centrales_P cp WHERE cp.codigo_central=codigo;
        
        --Si existe una violacin de la PK capturamos la excepcin para su posterior tratamiento.
        IF(existe > 0) THEN RAISE EX_CENTRAL_EXISTE; END IF;
        
	--Comprobamos que el resto de las variables, cuyo valor no puede ser nulo, tengan un valor, de no ser asi
        --capturamos las excepciones para su posterior tratamiento.
        IF(tipo is NULL)THEN RAISE EX_CENTRAL_TIPO_NULL; END IF;
        IF(dir is NULL)THEN RAISE EX_CENTRAL_DIRECCION_NULL; END IF;
        IF(loc is NULL)THEN RAISE EX_CENTRAL_LOCALIDAD_NULL; END IF;
        IF(ps is NULL)THEN RAISE EX_CENTRAL_PAIS_NULL; END IF;
    
	--Comprobamos que los atributos pasados por parmetro se correspoden con los del tipo de central
	IF(tipo='NUCLEAR' AND (consumo_func is NULL OR residuos_rad IS NULL)) THEN
		IF(emisiones_co2 IS NOT NULL OR molinos IS NOT NULL OR paneles IS NOT NULL)THEN RAISE EX_VALORES_NULOS_TIPO; END IF;
        END IF;
        
        IF((tipo='TERMICA' OR tipo='CARBO') AND (emisiones_co2 IS NULL)) THEN
		 IF(consumo_func IS NOT NULL OR residuos_rad IS NOT NULL OR molinos IS NOT NULL OR paneles IS NOT NULL) THEN RAISE EX_VALORES_NULOS_TIPO; END IF;
        END IF;
        
        IF(tipo='EOLICA' AND molinos IS NULL)THEN
		IF (emisiones_co2 IS NOT NULL OR consumo_func IS NOT NULL OR residuos_rad IS NOT NULL OR paneles IS NOT NULL)THEN RAISE EX_VALORES_NULOS_TIPO;END IF; 
        END IF;
        
        IF(tipo='SOLAR' AND paneles IS NULL) THEN
		IF (emisiones_co2 IS NOT NULL OR consumo_func IS NOT NULL OR residuos_rad IS NOT NULL OR molinos IS NOT NULL) THEN RAISE EX_VALORES_NULOS_TIPO; END IF;
        END IF;

        --Al no producirse ningn error realizamos la insercin
	INSERT INTO tb_Centrales_P VALUES(codigo,tipo,dir,loc,ps,consumo_func, residuos_rad, emisiones_co2, molinos, paneles, ultima_inspeccion,0.0);
        RSP:='OK';
        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
        
	--Ejecutamos Commit para confirmar la insercin definitiva en la base de datos.
        COMMIT;
        
    EXCEPTION
          WHEN EX_CENTRAL_EXISTE THEN 
            RSP:='ERROR: PRACP001: La central de produccin con el cdigo '|| codigo ||' ya ha sido dado de alta';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;
            
          WHEN EX_CENTRAL_TIPO_NULL THEN 
            RSP:='ERROR: PRACP002: El tipo de central de produccin '||codigo||' no puede tener un valor nulo';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;
                  
          WHEN EX_CENTRAL_DIRECCION_NULL THEN
             RSP:='ERROR: PRACP003: La direccin de la central de produccin '||codigo||' no puede tener un valor nulo';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;
                  
          WHEN EX_CENTRAL_LOCALIDAD_NULL THEN 
            RSP:='ERROR: PRACP004: La localidad de la central de produccin '||codigo||' no puede tener un valor nulo';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;
                   
          WHEN EX_CENTRAL_PAIS_NULL THEN 
            RSP:='ERROR: PRACP005: El pas de la central de produccin '||codigo||' no puede tener un valor nulo';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;
	
	  WHEN EX_VALORES_NULOS_TIPO THEN
	    RSP:='ERROR: PRACP006: Los valores pasados nos se corresponden con el tipo de central';
	    INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
         COMMIT;
                   
          WHEN OTHERS THEN 
            RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;
      END;
