--Alta contador--
/*Procedimiento que a partir de los parmetros de entrada aade un nuevo contador a la base de datos
 *PARAM IN: id_conta (variable de tipo NUMBER, contiene el identificador del contador)
 *	    direccion (variable de tipo Varchar2, contiene la direccin en la que est ubicado el contador)
 *	    localidad (variable de tipo Varchar2, contiene la localidad en la que se encuentra instalado el contador)
 *	    pais (variable de tipo Varchar2, contiene el pais de ubicacin del contador)
 *	    modelo (variable de tipo Varchar2, contiene el modelo del contador)
 *	    any_fab (variable de tipo NUMBER, contiene el ao de fabricacin del contador con todas sus cifras)
 *	    fabricante (variable de tipo Varchar2, contiene el nombre de la empresa fabricante del contador)
 *	    consumo_acumulado (variable de tipo NUMBER, contiene el valor de la ltima lectura realizada al contador)
 * 	    RSP (Variable de tipo CLOB que se corresponde con el parmetro de retorno).
  RETURN: Parmetro RSP que contendr informacin sobre el resultado de la ejecucin del procedimiento.
 */
CREATE OR REPLACE PROCEDURE pr_alta_contador(id_cont IN NUMBER, direccion IN VARCHAR2, localidad IN VARCHAR2, pais IN VARCHAR2, modelo IN VARCHAR2, any_fabricacion IN NUMBER, fabricante IN VARCHAR2, consumo_acumulado IN NUMBER, RSP IN OUT NOCOPY CLOB) AS
    
    --Declaracin de variables para la tabla de logs.
    fecha_proceso VARCHAR2(20);
    nombre_proceso VARCHAR2(20);
    In_log CLOB;
    
    --Variable de comprobacin
    existe INTEGER;
    
    --Declaracin de excepciones
    EX_CONTADOR_EXISTE EXCEPTION;
    EX_DIRECCION_NULL EXCEPTION;
    EX_LOCALIDAD_NULL EXCEPTION;
    EX_PAIS_NULL EXCEPTION;
    EX_MODELO_NULL EXCEPTION;
    EX_ANY_FABRICACION_NULL EXCEPTION;
    EX_FABRICANTE_NULL EXCEPTION;
    EX_CONSUMO_NULL EXCEPTION;
    
    --Ejecucin del procedimiento.
    BEGIN
          
	--Obtenemos la fecha de ejecucin del procedimiento a partir de los datos del sistema.
        fecha_proceso:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
        nombre_proceso:='pr_alta_contador';
      
        --Creamos una cadena de texto con los parametros de entrada.
        In_log:= (id_cont||'  ||  ' || direccion||'  ||  ' || localidad||'  ||  ' || pais||'  ||  ' ||modelo||'  ||  ' || any_fabricacion||'  ||  ' || fabricante||'  ||  ' || consumo_acumulado);        
        
	--Comprobamos que no existe un contador con la misma identificacin lo cual
        --supondra una violacin de la PK.
        SELECT COUNT(*)INTO existe FROM tb_Contadores c WHERE c.id_contador=id_cont;
        
        --Si existe una violacin de la PK capturamos la excepcin para su posterior tratamiento.
        IF(existe > 0) THEN RAISE EX_CONTADOR_EXISTE; END IF;
        --Comprobamos que el resto de las variables no tengan un valor nulo, de ser asi
        --capturamos las excepciones para su posterior tratamiento.
        IF(direccion is NULL)THEN RAISE EX_DIRECCION_NULL; END IF;
        IF(localidad is NULL)THEN RAISE EX_LOCALIDAD_NULL; END IF;
        IF(pais is NULL)THEN RAISE EX_PAIS_NULL; END IF;
        IF(modelo is NULL)THEN RAISE EX_MODELO_NULL; END IF;
        IF(any_fabricacion is NULL)THEN RAISE EX_ANY_FABRICACION_NULL; END IF;
        IF(fabricante is NULL)THEN RAISE EX_FABRICANTE_NULL; END IF;
        IF(consumo_acumulado is NULL)THEN RAISE EX_CONSUMO_NULL; END IF;
        
	--No se han producido excepciones por lo que realizamos la inserciones
	INSERT INTO tb_Contadores VALUES(id_cont, direccion, localidad, pais, modelo, any_fabricacion, fabricante, consumo_acumulado);
        RSP:='OK';
        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
   	
	--Ejecutamos commit para confirmar la insercin definitiva de los datos
	COMMIT;
	
    	--Tratamiento de excepciones

	EXCEPTION
          WHEN EX_CONTADOR_EXISTE THEN 
            RSP:='ERROR: PRACO001: El contador '|| id_cont ||' ya ha sido dado de alta';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
	  COMMIT;
                    
          WHEN EX_DIRECCION_NULL THEN
             RSP:='ERROR: PRACO002: La direccin del contador '||id_cont||' no puede tener un valor nulo';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
	  COMMIT;
                  
          WHEN EX_LOCALIDAD_NULL THEN 
            RSP:='ERROR: PRACO003: La localidad del contador '||id_cont||' no puede tener un valor nulo';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
	  COMMIT;
                    
          WHEN EX_PAIS_NULL THEN 
            RSP:='ERROR: PRACO004: El pas del contador '||id_cont||' no puede tener un valor nulo';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
	  COMMIT;   
         
          WHEN EX_MODELO_NULL THEN 
            RSP:='ERROR: PRACO005: El modelo del contador '||id_cont||' no puede tener un valor nulo';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
	  COMMIT;

          WHEN EX_ANY_FABRICACION_NULL THEN 
            RSP:='ERROR: PRACO006: El ao de fabricacin del contador '||id_cont||' no puede tener un valor nulo';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
	  COMMIT;

          WHEN EX_FABRICANTE_NULL THEN 
            RSP:='ERROR: PRACO007: El fabricante del contador '||id_cont||' no puede tener un valor nulo';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
	  COMMIT;

          WHEN EX_CONSUMO_NULL THEN 
            RSP:='ERROR: PRACO008: El consumo acumulado del contador '||id_cont||' no puede tener un valor nulo';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
	  COMMIT;           
 
          WHEN OTHERS THEN 
            RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
	  COMMIT;
      END;