--Alta contrato--
/*Procedimiento que a partir de los parmetros de entrada aade un nuevo contrato a la base de datos
 *PARAM IN: contrato (variable de tipo Varchar2, contiene el identificador nico del contrato)
 *	    alta (variable de tipo DATE, contiene la fecha de alta del contrato)
 *	    potencia_contratada (variable de tipo NUMBER, contiene la potencia contratada por el cliente)
 *	    cliente (variable de tipo Varchar2, contiene el identificador del cliente que firma el contrato)
 *	    contador (variable de tipo NUMBER contiene el identificador del contador asociado al contrato)
 *	    RSP (Variable de tipo CLOB que se corresponde con el parmetro de retorno).
 *RETURN: Parmetro RSP que contendr informacin sobre el resultado de la ejecucin del procedimiento.
 */
CREATE OR REPLACE
PROCEDURE pr_alta_contrato(contrato IN VARCHAR2, alta IN DATE,potencia_contratada IN NUMBER, cliente IN VARCHAR2, contador IN NUMBER, RSP OUT NOCOPY CLOB) AS
    
    --Declaracin de variables para la tabla de logs.
    fecha_proceso VARCHAR2(20);
    nombre_proceso VARCHAR2(20);
    In_log CLOB;
    
    --Variable de comprobacion
    existe_contrato INT;
    existe_contador INT;
    existe_cliente INT;
    contrato_activo INT;

    --Declaracin de excepciones
    EX_CONTRATO_EXISTE EXCEPTION;
    EX_FECHA_ALTA_NULL EXCEPTION;
    EX_CONTADOR_ASOCIADO EXCEPTION;
    EX_POTENCIA_NULL EXCEPTION;
    EX_CLIENTE EXCEPTION;
    EX_CONTADOR EXCEPTION;
    
    --Ejecucin del procedimiento.
    BEGIN
        
        fecha_proceso:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
        nombre_proceso:='pr_alta_contrato';
        --Creamos una cadena de texto con los parametros de entrada.
        In_log:= (contrato||'  ||  ' ||TO_CHAR(alta,'DD-MM-YY')||'  ||  '  || potencia_contratada||'  ||  ' ||cliente||'  ||  ' ||contador); 
       
        --Comprobamos que no existe un contrato con la misma identificacin lo cual
        --supondra una violacin de la PK.
        SELECT COUNT(*)INTO existe_contrato FROM tb_Contratos c WHERE c.id_contrato=contrato;

        --Si existe una violacin de la PK capturamos la excepcin para su posterior tratamiento.
        IF(existe_contrato > 0) THEN RAISE EX_CONTRATO_EXISTE; END IF;
       
	--Comprobamos que el resto de las variables no tengan un valor nulo, de ser asi
        --capturamos las excepciones para su posterior tratamiento.
        IF(alta is NULL)THEN RAISE EX_FECHA_ALTA_NULL; END IF;
        IF(potencia_contratada is NULL OR potencia_contratada=0)THEN RAISE EX_POTENCIA_NULL; END IF;
        
	--Comprobamos tambin que no exista violacin de la claves foraneas
        SELECT COUNT(*) INTO existe_cliente FROM tb_Clientes cli WHERE cli.id_cliente=cliente;
        SELECT COUNT(*) INTO existe_contador FROM tb_Contadores con WHERE con.id_contador=contador;
        SELECT COUNT(*) INTO contrato_activo FROM tb_Contratos co WHERE co.contador=contador AND fecha_baja IS NOT NULL;
        
        IF(cliente is NULL OR existe_cliente=0)THEN RAISE EX_CLIENTE; END IF;
        IF(contador is NULL OR existe_contador=0)THEN RAISE EX_CONTADOR; END IF;
        IF(contrato_activo >0) THEN RAISE EX_CONTADOR_ASOCIADO;END IF;
 
        --No se existen excepciones por lo que realizamos las inserciones
	INSERT INTO tb_Contratos VALUES(contrato, alta, null ,potencia_contratada,cliente,contador);
        RSP:='OK';
        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
	
	--Ejecutamos commit para confirmar la inserccin definitiva en la BD.
	COMMIT;   

    --Tratamiento de excepciones
   
    EXCEPTION
          WHEN EX_CONTRATO_EXISTE THEN 
            RSP:='ERROR: PRACN001: El contrato '|| contrato ||' ya ha sido dado de alta';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;
          
          WHEN EX_FECHA_ALTA_NULL THEN
             RSP:='ERROR: PRACN002: La fecha de alta del contrato '||contrato||' no puede tener un valor nulo';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;
                  
          WHEN EX_POTENCIA_NULL THEN 
            RSP:='ERROR: PRACN003: El valor de la potencia contratada en el contrato '||contrato||' no puede ser nulo o cero';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);                    
          COMMIT; 

          WHEN EX_CLIENTE THEN 
            RSP:='ERROR: PRACN004: El NIF del cliente asociado al contrato '||contrato||' no existe o su valor es nulo';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;
            
          WHEN EX_CONTADOR THEN 
            RSP:='ERROR: PRACN005: El identificador del contador asociado al contrato '||contrato||' no existe o su valor es nulo';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;            

          WHEN EX_CONTADOR_ASOCIADO THEN 
            RSP:='ERROR: PRACN006: El contador ya esta asociado a un contrato en activo';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;
            
          WHEN OTHERS THEN 
            RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;

      END;
