--Alta lectura
/*Procedimiento que a partir de los parmetros de entrada aade una nueva lectura a la base de datos
 *PARAMETROS IN: id_conta (Variable de tipo NUMBER, contiene el identificador nico del contador)
 *		 fecha_lect (Variable de tipo DATE, contiene la fecha de la lectura del contador)
 *		 modalidad_lectura (Variable de tipo Varchar2, contiene el tipo de lectura (TELEMATICA o PRESENCIAL).
 *		 lectura (Variable de tipo NUMBER, contiene el valor de la lectura del contador) 
 *	         RSP (Variable de tipo CLOB que se corresponde con el parmetro de retorno).
 *RETURN: Parmetro RSP que contendr informacin sobre el resultado de la ejecucin del procedimiento.
 */

CREATE OR REPLACE
PROCEDURE pr_alta_lectura(id_conta IN NUMBER, fecha_lect IN DATE, modalidad_lectura IN VARCHAR, lectura IN NUMBER, RSP OUT NOCOPY CLOB) AS
    
    --Declaracin de variables para la tabla de logs.
    fecha_proceso VARCHAR2(20);
    nombre_proceso VARCHAR2(20);
    In_log CLOB;
    
    --Variables de comprobacion
    consumo_mensual NUMBER(6,2);
    consumo_anterior NUMBER(10,2);
    existe_lectura INT;
    existe_contador INT;
    
    --Declaracin de excepciones
    EX_LECTURA_EXISTE EXCEPTION;
    EX_LECTURA_ERRONEA EXCEPTION;
    EX_CONTADOR EXCEPTION;
    EX_FECHA EXCEPTION;
    EX_MODALIDAD_NULL EXCEPTION;
    EX_LECTURA_NULL EXCEPTION;   
    
    --Ejecucin del procedimiento.
    BEGIN
 
        --Obtenemos la fecha de ejecucin del procedimiento de los datos del sistema.
	fecha_proceso:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
        nombre_proceso:='pr_alta_lectura';

	--Creamos una cadena de texto con los parametros de entrada.
        In_log:= (id_conta||'  ||  ' ||TO_CHAR(fecha_lect,'DD-MM-YYYY')||'  ||  ' ||modalidad_lectura||'  ||  ' || lectura); 
                
        --Comprobamos que la lectura no es nula.
	IF(lectura IS NULL)THEN RAISE EX_LECTURA_NULL; END IF;
        
        --Comprobamos que la lectura no ha sido dada de alta en la base de datos
	SELECT COUNT(*)INTO existe_lectura FROM tb_Lecturas lec WHERE (lec.contador=id_conta AND lec.fecha=fecha_lect);
        --Si existe una violacin de la PK capturamos la excepcin para su posterior tratamiento.
        IF(existe_lectura > 0) THEN RAISE EX_LECTURA_EXISTE; END IF;
        
        --Comprobamos que el valor de la lectura sea mayor o igual al valor de la lectura anterior
	SELECT consumo_acumulado INTO consumo_anterior FROM tb_Contadores con WHERE con.id_contador=id_conta;
        IF(consumo_anterior > lectura) THEN RAISE EX_LECTURA_ERRONEA;END IF;
    

        --Comprobamos que el resto de las variables no tengan un valor nulo, de ser asi
        --capturamos las excepciones para su posterior tratamiento.
        IF(fecha_lect is NULL)THEN RAISE EX_FECHA; END IF;
        IF(modalidad_lectura is NULL)THEN RAISE EX_MODALIDAD_NULL; END IF;

        --Comprobamos tambin que no exista violacin de la clave foranea
        SELECT COUNT(*) INTO existe_contador FROM tb_Contadores con WHERE con.id_contador=id_conta;
        IF(id_conta is NULL OR existe_contador=0)THEN RAISE EX_CONTADOR; END IF;
        
        --No existen excepciones por lo que realizamos las inserciones.
	consumo_mensual:=lectura-consumo_anterior;
        INSERT INTO tb_Lecturas VALUES(id_conta, fecha_lect, modalidad_lectura ,lectura,consumo_mensual);
        
        --actualizamos el consumo acumulado en el contador
        UPDATE tb_Contadores con SET con.consumo_acumulado=lectura WHERE con.id_contador=id_conta;
        RSP:='OK';
        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);

	--ejecutamos commit para confirmar la inserccin definitiva en la BD.
	COMMIT;   

   --Tratamiento de excepciones

    EXCEPTION
          WHEN EX_LECTURA_EXISTE THEN 
            RSP:='ERROR: PRALC001: La lectura del contador '|| id_conta ||' en fecha ' ||fecha_lect|| ' ya ha  sido introducida en la base de datos';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;
          
          WHEN EX_LECTURA_NULL THEN
             RSP:='ERROR: PRALC002: El valor de la lectura del contador '||id_conta ||' en fecha ' ||fecha_lect||' no puede ser nulo';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;          
        
          WHEN EX_LECTURA_ERRONEA THEN 
            RSP:='ERROR: PRALC003: El valor de la lectura del contador '||id_conta ||' en fecha ' ||fecha_lect||' no puede ser inferior al valor de la lectura del mes anterior';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);                    
          COMMIT; 

          WHEN EX_MODALIDAD_NULL THEN 
            RSP:='ERROR: PRALC004: La modalidad de lectura del contador '||id_conta ||' en fecha ' ||fecha_lect||' no puede ser nula';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;
            
          WHEN EX_CONTADOR THEN 
            RSP:='ERROR: PRALC005: El identificador del contador asociado a la lectura  no existe o su valor es nulo';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;            

          WHEN EX_FECHA THEN 
            RSP:='ERROR: PRALC006: El valor de la fecha de la lectura no puede ser nulo';
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;
            
          WHEN OTHERS THEN 
            RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
            INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
          COMMIT;
      END;
