/*Procedimiento que a partir una central de produccin nos devuelve el consumo de los contadores conectados a 
 *ella y la enega total producida para un periodo de tiempo dado.
 *ciudad dada en un mes determinado
 *PARAMETROS IN: id_central (Variable de tipo VARCHAR2, contiene el cdigo de la central a consultar)
 *		 fecha_inicial(Variable de tipo DATE, contiene la fecha inicial del periodo a consultar)
 *               fecha_final (Variable de tipo DATE, cotiene la fecha final del periodo a consultar)
 *               CURSOR_R (Variable de tipo SYS_REFCURSOR, que se corresponde con el parmetro de retorno que contendr
 *			   los datos solicitados)
 *		 RSP (Variable de tipo CLOB que se corresponde con el parmetro de retorno).
 *RETURN: 	 CURSOR_R (Contendr la respuesta con el consumo total de los contadores y la produccin de la central
 *			   para el periodo dado y en este orden).
 *		 Parmetro RSP que contendr informacin sobre el resultado de la ejecucin del procedimiento.
 */


CREATE OR REPLACE
PROCEDURE pr_consulta_e(id_central IN VARCHAR2, fecha_inicial IN DATE, fecha_final IN DATE, CURSOR_R OUT NOCOPY SYS_REFCURSOR, RSP OUT NOCOPY CLOB) AS

    --Declaracin de variables para la tabla Log
    fecha_proceso VARCHAR2(20);
    nombre_proceso VARCHAR2(35);
    In_log CLOB;

    --Declaracin de variables 
    mes_inicial INTEGER;
    any_inicial INTEGER;
    mes_final INTEGER;
    any_final INTEGER;
    consumo_contadores NUMBER(10,2);
    produccion_central NUMBER(10,2);	    

    --Variable de comprobacion
    existe_central INTEGER;
    existe_lectura INTEGER;
    
    --Excepciones
    EX_CENTRAL_NO_EXISTE EXCEPTION;
    EX_LECTURA_NO_EXISTE EXCEPTION;
    EX_ERROR_FECHA EXCEPTION;
   
    --Iniciamos
    BEGIN
                     
        fecha_proceso:= TO_CHAR(SYSTIMESTAMP,'DD-MM-YYYY HH24:MI:SS');
        nombre_proceso:='pr_consulta_e';
      
        --Creamos una cadena de texto con los parametros de entrada.

        In_log:= (id_central||'  ||  '||TO_CHAR(fecha_inicial,'DD/MM/YYYY')||'  ||  ' || TO_CHAR(fecha_final,'DD/MM/YYYY'));  
        
        --Comprobamos que la central existe
	SELECT COUNT(*) INTO existe_central FROM tb_Centrales_P WHERE codigo_central=id_central;
	IF(existe_central=0) THEN RAISE EX_CENTRAL_NO_EXISTE; END IF;

	--Comprobamos que la fecha final del periodo no es menor que la fecha inicial
	IF(To_Date(fecha_final)<to_Date(fecha_inicial)) THEN RAISE EX_ERROR_FECHA; END IF;
	
	--Comprobamos que existen lecturas en el intervalo de fechas pasado por parametro.
        SELECT COUNT(*)INTO existe_lectura FROM tb_Lecturas tl 
        WHERE tl.fecha BETWEEN TO_DATE(fecha_inicial) AND TO_DATE( fecha_final);
        IF(existe_lectura=0)THEN RAISE EX_LECTURA_NO_EXISTE; END IF;

	--Obtenemos el mes y ao de cada una de las fechas
        SELECT EXTRACT(MONTH FROM fecha_inicial)INTO mes_inicial FROM DUAL;
	SELECT EXTRACT(YEAR FROM fecha_inicial)INTO any_inicial FROM DUAL;

	SELECT EXTRACT(MONTH FROM fecha_final)INTO mes_final FROM DUAL;
	SELECT EXTRACT(YEAR FROM fecha_final)INTO any_final FROM DUAL;
       --Abrimos el cursor
       
        OPEN CURSOR_R FOR
		SELECT
                      (SELECT SUM(TLE.CONSUMO_MES/((
				SELECT COUNT(*) FROM TB_DISTRIBUCION TD WHERE
				TD.CONTADOR=TLE.CONTADOR)*(
				SELECT COUNT(*) FROM TB_ALIMENTACION TA 
				WHERE TA.DIRECCION=TDI.DIRECCION AND TA.LOCALIDAD=TDI.LOCALIDAD AND TA.PAIS=TDI.PAIS)))
			FROM TB_LECTURAS TLE INNER JOIN(
			        TB_DISTRIBUCION TDI INNER JOIN(
			            TB_ALIMENTACION TAL INNER JOIN TB_LINEAS TLI 
			            ON TAL.ID_LINEA=TLI.CODIGO_LINEA)
			        ON TDI.DIRECCION=TAL.DIRECCION) 
			     ON TLE.CONTADOR=TDI.CONTADOR
			WHERE TLI.CENTRAL_P=id_central AND TLE.FECHA BETWEEN TO_DATE(fecha_inicial)AND TO_DATE(fecha_final))
			AS "CONSUMO_CONTADORES"
			,
			(SELECT SUM(PRODUCCION)FROM TB_HISTORICO 
			WHERE CENTRAL_P=id_central 
			AND (ANYO*12+MES) BETWEEN (any_inicial*12+mes_inicial)AND (any_final*12+mes_final)) 
			AS "PRODUCCION_CENTRAL" 
		FROM DUAL;
        
        --Recorremos el cursor para comprobar resultados 
        LOOP FETCH CURSOR_R INTO consumo_contadores, produccion_central;
        EXIT WHEN CURSOR_R%NOTFOUND;  
		dbms_output.put_line(consumo_contadores ||'   '||produccion_central);
        END LOOP;
	        
        RSP:='OK';
        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,'cursor con los datos solicitados',RSP); 
        COMMIT;
 
     EXCEPTION

	WHEN EX_CENTRAL_NO_EXISTE THEN
        RSP:='ERROR: PRCOE001: La Central de produccin no existe en la BD, compruebe el cdigo introducido';
        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);

	WHEN EX_ERROR_FECHA THEN
        RSP:='ERROR: PRCOE002: La fecha final de consulta no puede ser menor que la fecha incial';
        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);        
        
	WHEN EX_LECTURA_NO_EXISTE THEN
        RSP:='ERROR: PRCOF003: No existen lecturas en el intervalo de tiempo dado';
        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
        
        WHEN OTHERS THEN 
        RSP:='ERROR: ' || SQLCODE || ' ' || SQLERRM;  
        INSERT INTO tb_Logs VALUES (seq_id_logs.NextVal,fecha_proceso,nombre_proceso,In_log,NULL,RSP);
    END;
